package org.qtunes.auth.control;

import org.qtunes.core.*;
import org.qtunes.auth.*;
import org.qtunes.db.*;
import java.util.*;
import java.io.*;
import org.qtunes.core.util.JSONSerializer;

/**
 * A JSONSerializer that has been customizer to serialize/deserialize
 * object likely to to passed to or from the ControlMethodInvoker
 */
public class ControlJSONSerializer extends JSONSerializer {

    private Collection features;

    public ControlJSONSerializer() {
        this.features = Collections.EMPTY_SET;
    }

    /**
     * Features:
     * bitsetmap - bitset serialized as {bitset: fe00000 }
     */
    public ControlJSONSerializer(String[] features) {
        this.features = Arrays.asList(features);
    }

    public Object getCustomWrite(Object o) {
        if (o instanceof BitSet) {
            BitSet bitset = (BitSet)o;
            if (features.contains("bitsetmap")) {
                Map<String,String> m = new HashMap<String,String>();
                StringBuffer sb = new StringBuffer();
                int bit = (bitset.length() & ~3);
                do {
                    int v = 0;
                    if (bitset.get(bit+3)) v += 8;
                    if (bitset.get(bit+2)) v += 4;
                    if (bitset.get(bit+1)) v += 2;
                    if (bitset.get(bit+0)) v += 1;
                    sb.append(Integer.toString(v, 16));
                    bit -= 4;
                } while (bit>=0);
                m.put("bitset", sb.toString());
                return m;
            } else {
                int[] values = new int[bitset.cardinality()];
                int j = 0;
                for (int i=bitset.nextSetBit(0); i>=0; i=bitset.nextSetBit(i+1)) {
                    values[j++] = i;
                }
                return values;
            }
        } else if (o instanceof Session) {
            Map<String,Object> m = new LinkedHashMap<String,Object>();
            m.put("id", Integer.valueOf(((Session)o).getId()));
            m.put("allowances", new ArrayList<String>(((Session)o).getAllowances()));
            m.put("props", ((Session)o).getProperties());
            return m;
        } else if (o instanceof java.net.InetSocketAddress) {
            return o.toString();
        } else if (o instanceof SmartPlaylist) {
            return ((SmartPlaylist)o).getQuery();
        } else if (o instanceof Playlist) {
            return ((Playlist)o).getTracks();
        } else if (o instanceof Track) {
            return ((Track)o).getIndex();
        } else if (o instanceof File) {
            try {
                return ((File)o).getCanonicalPath();
            } catch (IOException e) {
                return ((File)o).getAbsolutePath();
            }
        } else if (o instanceof Service) {
            return ((Service)o).getContext().getServiceName();
        } else if (o instanceof Class) {
            return ((Class)o).getName();
        } else if (o instanceof Throwable) {
            Throwable t = (Throwable)o;
            Map<String,Object> e = new LinkedHashMap<String,Object>();
            e.put("type", t.getClass().getName());
            if (t.getMessage()!=null && t.getMessage().length() > 0) {
                e.put("message", t.getMessage());
            }
            StackTraceElement[] elt = t.getStackTrace();
            List<String> outelt = new ArrayList<String>();
            for (int i=0;i<elt.length;i++) {
                StringBuffer s = new StringBuffer();
                s.append(elt[i].getClassName()+"."+elt[i].getMethodName());
                String file = elt[i].getFileName();
                if (file!=null) {
                    s.append("("+file);
                    if (elt[i].getLineNumber() >= 0) {
                        s.append(":"+elt[i].getLineNumber());
                    }
                    s.append(")");
                }
                outelt.add(s.toString());
            }
            e.put("trace", outelt);
            if (t.getCause()!=null) {
                e.put("cause", getCustomWrite(t.getCause()));
            }
            return e;
        } else {
            System.out.println(o.getClass());
            return super.getCustomWrite(o);
        }
    }

    public Object getCustomRead(Map m) {
        if (features.contains("bitsetmap") && m.size()==1 && m.containsKey("bitset")) {
            String s = (String)m.get("bitset");
            int bit = s.length() * 4;
            BitSet bitset = new BitSet(bit+4);
            for (int i=0;i<s.length();i++) {
                char c = s.charAt(i);
                if (c>='a' || c<='f') {
                    c-='a'-10;
                } else if (c>='A' || c<='F') {
                    c-='A'-10;
                }
                if ((c&8)!=0) bitset.set(bit+3);
                if ((c&4)!=0) bitset.set(bit+2);
                if ((c&2)!=0) bitset.set(bit+1);
                if ((c&1)!=0) bitset.set(bit+0);
                bit -= 4;
            }
            return bitset;
        } else {
            return super.getCustomRead(m);
        }
    }

}
