package org.qtunes.auth.control;

import org.qtunes.core.*;
import org.qtunes.auth.*;
import org.qtunes.db.*;
import org.qtunes.core.util.JSONSerializer;
import java.lang.reflect.Array;
import java.util.*;
import java.io.IOException;

/**
 * A ParamParser that parses it's input as JSON.
 *
 * {object:db, calls:[{method:"getPlaylist", args:{name:"MyList"}}, {method:"addTrack", args:{track: 24}}]}
 */
public class JSONParser implements ParamParser {

    protected final ServiceContext context;
    protected final Auth auth;
    protected final JSONSerializer json;
    protected List<Call> calls;
    protected Session session;
    protected String object;

    public JSONParser(ServiceContext context) {
        this.context = context;
        this.auth = context.getService(Auth.class);
        this.json = new ControlJSONSerializer(new String[] { "bitsetmap" });
    }

    @SuppressWarnings("unchecked")
    public void parse(String in) throws Exception {
        Map values = (Map)json.read(in);
        this.object = (String)values.get("object");
        if (object==null) {
            throw new IllegalArgumentException("No object");
        }
        List l = (List)values.get("calls");
        if (l==null) {
            throw new IllegalArgumentException("No calls");
        }
        this.calls = new ArrayList<Call>(l.size());
        for (int i=0;i<l.size();i++) {
            Map m = (Map)l.get(i);
            String method = (String)m.get("method");
            Map<String,Object> args = (Map<String,Object>)m.get("args");
            if (args==null) {
                args = Collections.emptyMap();
            }
            if (method==null) {
                throw new IllegalArgumentException("No method name");
            }
            this.calls.add(new Call(method, args));
        }
        Integer sessionid = (Integer)values.get("session");
        this.session = sessionid == null ? null : auth.getSession(sessionid.intValue());
    }

    public String getObjectName() {
        return object;
    }

    public List<Call> getMethodCalls() {
        return calls;
    }

    public Session getSession() {
        return session;
    }

    public String toString(Object o) {
        if (o==Void.TYPE || o==null) {
            return "{ok:true}";
        } else if (!(o instanceof Throwable)) {
            try {
                return "{ok:true, response:"+json.write(o)+"}";
            } catch (Throwable e) {
                o = e;
            }
        }
        try {
            Map<String,Object> m = new LinkedHashMap<String,Object>();
            m.put("ok", Boolean.FALSE);
            m.put("exception", o);
            m.put("sessionok", Session.getCurrentSession()!=Session.NONE);
            return json.write(m);
        } catch (Throwable e) {
            e.printStackTrace();
            return "{ok:false}";
        }
    }

}
