package org.qtunes.core.util;

import java.util.*;
import java.io.*;
import java.net.*;

/**
 * Utilty class to parse the parameters in GET url, this adds functionality
 * surprisingly absent from com.sun.httpserver package.
 */
public class HttpParameters {

    private Map<String, Object> parameters;

    public HttpParameters(URI uri, String encoding) throws UnsupportedEncodingException {
        this(uri.getRawQuery(), encoding);
    }

    @SuppressWarnings("unchecked")
    public HttpParameters(String rawquery, String encoding) throws UnsupportedEncodingException {
        parameters = new HashMap<String, Object>();
        if (rawquery!=null) {
            StringTokenizer st = new StringTokenizer(rawquery, "&");
            while (st.hasMoreTokens()) {
                String kv = st.nextToken();
                StringTokenizer st2 = new StringTokenizer(kv, "=");
                String key = null;
                String value = "";
                if (st2.hasMoreTokens()) {
                    key = st2.nextToken();
                    key = URLDecoder.decode(key, encoding);
                    if (st2.hasMoreTokens()) {
                        value = st2.nextToken();
                        value = URLDecoder.decode(value, encoding);
                    }
                    if (parameters.containsKey(key)) {
                        Object o = parameters.get(key);
                        if (o instanceof List) {
                            List<String> values = (List<String>) o;
                            values.add(value);
                        } else if (o instanceof String) {
                            List<String> values = new ArrayList<String>();
                            values.add((String)o);
                            values.add(value);
                            parameters.put(key, values);
                        }
                    } else {
                        parameters.put(key, value);
                    }
                }
            }
        }
    }

    @SuppressWarnings("unchecked")
    public String getValue(String key) {
        Object o = parameters.get(key);
        if (o instanceof String) {
            return (String)o;
        } else if (o instanceof List) {
            return ((List<String>)o).get(0);
        } else {
            return null;
        }
    }

    @SuppressWarnings("unchecked")
    public List<String> getValues(String key) {
        Object o = parameters.get(key);
        if (o instanceof String) {
            return Collections.singletonList((String)o);
        } else if (o instanceof List) {
            return (List<String>)o;
        } else {
            return null;
        }
    }

    public Map<String,Object> getAllValues() {
        return parameters;
    }

    public void remove(String key) {
        parameters.remove(key);
    }

    public boolean containsKey(String key) {
        return parameters.containsKey(key);
    }

}
