package org.qtunes.daap;

import org.qtunes.db.*;
import org.qtunes.web.*;
import java.util.*;
import java.io.*;

class HandlerContainerItems extends AbstractHandler {

    Block handleDAAP(DaapServer daap, WebConnection con) throws IOException {
        int container = Integer.parseInt(con.getParameter("container"));
        String[] meta = (con.getParameter("meta")).split(",");
        String mediatype = con.getParameter("type");
        Database db = daap.getDatabase();
        SmartPlaylist filter = DaapPlaylist.getSmartPlaylist(con.getParameter("query"), db);
        List<Track> filtertracks = filter==null ? null : filter.getTracks();

        ListBlock apso = Blocks.create(Blocks.apso);
        apso.add(Blocks.create(Blocks.mstt, 200));
        apso.add(Blocks.create(Blocks.muty, 0));

        ListBlock mlcl = Blocks.create(Blocks.mlcl);
        DaapPlaylist apl = daap.getPlaylist(container);
        if (apl!=null) {
            Playlist playlist = apl.getPlaylist();
            List<Track> tracks = playlist.getTracks();
            for (int i=0;i<tracks.size();i++) {
                Track track = tracks.get(i);
                if (filtertracks==null || filtertracks.contains(track)) {
                    ListBlock mlit = Blocks.create(Blocks.mlit);
                    mlcl.add(mlit);
                    for (int j=0;j<meta.length;j++) {
                        String longdesc = meta[j];
                        Block block = Blocks.getFieldAsBlock(longdesc, track);
                        if (block!=null) {
                            mlit.add(block);
                        }
                    }
                    mlit.add(Blocks.create(Blocks.mcti, mlcl.size()));
                }
            }
            apso.add(Blocks.create(Blocks.mtco, mlcl.size()));
            apso.add(Blocks.create(Blocks.mrco, mlcl.size()));
            apso.add(mlcl);
            return apso;
        } else {
            con.sendResponseHeaders(404, 0);
            return null;
        }
    }

    @Override String getCacheKey(WebConnection con) {
        String container = con.getParameter("container");
        String[] meta = (con.getParameter("meta")).split(",");
        String mediatype = con.getParameter("type");
        String query = con.getParameter("query");
        return "containeritems."+mediatype+"."+container+"."+query+"."+Arrays.asList(meta);
    }

}
