package org.qtunes.daap;

import org.qtunes.db.*;
import org.qtunes.web.*;
import org.qtunes.player.*;
import java.io.*;
import java.nio.ByteBuffer;
import java.util.*;

class HandlerGetProperty extends AbstractHandler {

    private static final int PAUSED=3, STOPPED=2, PLAYING=4;

    Block handleDAAP(DaapServer daap, WebConnection con) throws IOException {
        String[] properties = (con.getParameter("properties")).split(",");
        return handleGetProperties(properties, Blocks.cmgt, daap);
    }

    public static Block handleGetProperties(String[] properties, Blocks.BlockDefL listdef, DaapServer daap) {
        ListBlock cmgt = Blocks.create(listdef);
        cmgt.add(Blocks.create(Blocks.mstt, 200));

        Player player = daap.getPlayer();
        Track track = player.getCurrentTrack();
        int trackindex = player.getCurrentTrackIndex();

        for (int i=0;i<properties.length;i++) {
            Blocks.BlockDef def = Blocks.getFromDescription(properties[i]);
            if (properties[i].equals("dmcp.serverrevision")) {                  // cmsr
                Revision revision = daap.getPlayRevision();
                cmgt.add(Blocks.create((Blocks.BlockDefN)def, revision.getRevision()));
            } else if (properties[i].equals("dacp.playerstate")) {              // caps
                boolean paused = player.isPaused();
                cmgt.add(Blocks.create((Blocks.BlockDefN)def, track == null ? STOPPED : paused ? PAUSED : PLAYING));
            } else if (properties[i].equals("dacp.playingtime")) {              // cant
                if (track!=null) {
                    int time = ((Number)track.get(Field.Duration)).intValue();
                    time -= player.getTrackPosition();
                    cmgt.add(Blocks.create((Blocks.BlockDefN)def, time));
                }
            } else if (properties[i].equals("dmcp.volume")) {                   // cmvo
                cmgt.add(Blocks.create((Blocks.BlockDefN)def, Math.round(player.getVolume())));
            } else if (properties[i].equals("dacp.availableshufflestates")) {   // caas
                cmgt.add(Blocks.create((Blocks.BlockDefN)def, 2));
            } else if (properties[i].equals("dacp.volumecontrollable")) {       // cavc
                cmgt.add(Blocks.create((Blocks.BlockDefN)def, 1));
            } else if (properties[i].equals("dacp.availablerepeatstates")) {    // caar
                cmgt.add(Blocks.create((Blocks.BlockDefN)def, 6));
            } else if (properties[i].equals("dacp.shufflestate")) {             // cash
                cmgt.add(Blocks.create((Blocks.BlockDefN)def, player.getShuffle() ? 1 : 0));
            } else if (properties[i].equals("dacp.repeatstate")) {
                cmgt.add(Blocks.create((Blocks.BlockDefN)def, player.getRepeat()));
            } else if (properties[i].equals("dacp.nowplaying")) {
                if (track!=null) {
                    DaapPlaylist playlist = daap.getPlaylist("Now Playing");
                    ByteBuffer buf = ByteBuffer.wrap(new byte[16]);
                    buf.putInt(1);                                              // databaseid
                    buf.putInt(playlist.getIndex());                            // containerid
                    buf.putInt(playlist.getCIDIndex(trackindex));               // containeritemid
                    buf.putInt(track.getIndex());                               // itemid
                    cmgt.add(Blocks.create(Blocks.canp, buf.array()));
                    String title =  (String)track.get(Field.Title);
                    String artist = (String)track.get(Field.Artist);
                    String album =  (String)track.get(Field.Album);
                    long albumid = ((Long)track.get(DaapField.AlbumId)).longValue();
                    String genre =  (String)track.get(Field.Genre);
                    cmgt.add(Blocks.create(Blocks.cann, title));
                    if (artist!=null) {
                        cmgt.add(Blocks.create(Blocks.cana, artist));
                    }
                    if (album!=null) {
                        cmgt.add(Blocks.create(Blocks.canl, album));
                    }
                    if (genre!=null) {
                        cmgt.add(Blocks.create(Blocks.cang, genre));
                    }
                    if (albumid!=0) {
                        cmgt.add(Blocks.create(Blocks.asai, albumid));
                    }

                    cmgt.add(Blocks.create(Blocks.cmmk, 1));
                    cmgt.add(Blocks.create(Blocks.cast, ((Number)track.get(Field.Duration)).intValue()));
                }
            } else {
                throw new IllegalStateException("Unknown property "+properties[i]);
            }
        }
        return cmgt;
    }

}
