package org.qtunes.daap;

import org.qtunes.db.*;
import org.qtunes.web.*;
import java.util.*;
import java.io.*;

class HandlerItems extends AbstractHandler {

    Block handleDAAP(DaapServer daap, WebConnection con) throws IOException {
        String[] meta = (con.getParameter("meta")).split(",");
        if (meta.length==1 && meta[0].equals("all")) {
            meta = Blocks.getAllMeta();
        }
        String mediatype = con.getParameter("type");
        Database db = daap.getDatabase();
        List<Track> tracks = db.getPlaylist("*").getTracks();

        ListBlock adbs = Blocks.create(Blocks.adbs);      // Note, tapjam docs incorrecty has "apso".
        adbs.add(Blocks.create(Blocks.mstt, 200));
        adbs.add(Blocks.create(Blocks.muty, 0));

        ListBlock mlcl = Blocks.create(Blocks.mlcl);
        for (int i=0;i<tracks.size();i++) {
            Track track = tracks.get(i);
            ListBlock mlit = Blocks.create(Blocks.mlit);
            mlcl.add(mlit);
            if (Arrays.asList(meta).contains("dmap.itemkind")) {        // mikd must be first
                Block block = Blocks.getFieldAsBlock("dmap.itemkind", track);
                mlit.add(block);
            }
            for (int j=0;j<meta.length;j++) {
                String longdesc = meta[j];
                if (!longdesc.equals("dmap.itemkind")) {
                    Block block = Blocks.getFieldAsBlock(longdesc, track);
                    if (block!=null) {
                        mlit.add(block);
                    }
                }
            }
        }

        adbs.add(Blocks.create(Blocks.mtco, mlcl.size()));
        adbs.add(Blocks.create(Blocks.mrco, mlcl.size()));
        adbs.add(mlcl);
        return adbs;
    }

    @Override String getCacheKey(WebConnection con) {
        String[] meta = (con.getParameter("meta")).split(",");
        String mediatype = con.getParameter("type");
        return "items."+mediatype+"."+Arrays.asList(meta);
    }

}
