package org.qtunes.daap;

import java.io.*;

/**
 * A type of Block that represents a 1, 2, 4 or 8 byte number
 */
class NumericBlock extends Block {

    private long value;
    private byte type;

    public String toString() {
        return getName()+"="+get();
    }

    void setType(int type) {
        if (type==Blocks.VER) type = Blocks.INT;
        if (type==Blocks.DAT) type = Blocks.INT;
        this.type = (byte)type;
    }

    void set(long value) {
        this.value = value;
    }

    long get() {
        return this.value;
    }

    int length() {
        return 1 << (((type+1)/2)-1);     // 1, 2, 4 or 8
    }

    void read(InputStream in, int length) throws IOException {
        long v = 0;
        length = length();      // Required?
        for (int i=length-1;i>=0;i--) {
            v = (v<<8) + in.read();
        }
        set(v);
    }

    void writeContent(OutputStream out) throws IOException {
        for (int i=length()-1;i>=0;i--) {
            out.write((int)(value>>(i*8)));
        }
    }

    void dump(Appendable sb, String prefix) throws IOException {
        sb.append(getName()+"="+get()+"("+length()+")");
    }

}
