package org.qtunes.db;

import org.qtunes.core.*;
import java.util.*;
import java.io.*;

/**
 * A Simple Playlist that has a manually managed list of tracks.
 * The list may contain duplicates and the order is, like any
 * list, the order in which the items are added.
 */
public interface ManualPlaylist extends Playlist {

    /**
     * Add the tracks from the specified Playlist
     * @param playlist the tracks to add
     */
    @ControlMethod (
        params = {"playlist"},
        help = "Add the tracks from the specified Playlist"
    )
    public void addAll(Playlist playlist);

    /**
     * Add the specified tracks to this Playlist, but only if they're not duplicates.
     * @param playlist the tracks to add
     */
    @ControlMethod (
        params = {"playlist"},
        help = "Add non-duplicate tracks from the specified Playlist"
    )
    public void mergeAll(Playlist playlist);

    /**
     * Add the specified tracks to this Playlist at the specified point
     * @param playlist the tracks to add
     * @param index the insertion position in the list
     */
    @ControlMethod (
        params = {"playlist", "index"},
        help = "Add the tracks from the specified Playlist at the specified position"
    )
    public void addAll(Playlist playlist, int index);

    /**
     * Add the specified tracks to this Playlist at the specified point,
     * but only if they're not duplicates
     * @param playlist the tracks to add
     * @param index the insertion position in the list
     */
    @ControlMethod (
        params = {"playlist", "index"},
        help = "Add non-duplicate tracks from the specified Playlist at the specified position"
    )
    public void mergeAll(Playlist playlist, int index);

    /**
     * Add the specified Track to the Playlist.
     * @param track the Track to add
     */
    @ControlMethod (
        params = {"track"},
        help = "Add the specified Track to the Playlist"
    )
    public void add(Track track);

    /**
     * Add the specified Track to the Playlist.
     * @param track the Track to add
     * @param index the insertion position in the list
     */
    @ControlMethod (
        params = {"track","position"},
        help = "Add the specified Track to the Playlist at the specified position"
    )
    public void add(Track track, int index);

    /**
     * Clear the Playlist
     */
    @ControlMethod (
        params = {},
        help = "Clear the Playlist"
    )
    public void clear();

    /**
     * Remove the Track at the specified position in the Playlist
     */
    @ControlMethod (
        params = {"index"},
        help = "Delete the Track at the specified position"
    )
    public Track remove(int index);

    /**
     * Remove all occurances of the specified Track from the Playlist
     */
    @ControlMethod (
        params = {"track"},
        help = "Remove all occurances of the Track from the playlist"
    )
    public boolean removeTrack(Track track);

    /**
     * Remove all tracks from the specified Playlist from this Playlist
     */
    @ControlMethod (
        params = {"playlist"},
        help = "Remove all tracks from the specified Playlist from this Playlist"
    )
    public void removeAll(Playlist playlist);

    /**
     * Remove tracks at the specified indices from this Playlist
     */
    @ControlMethod (
        params = {"playlist"},
        help = "Remove tracks at the specified indices from this Playlist"
    )
    public void removeAll(BitSet indices);
}
