package org.qtunes.db.spi.simple;

import org.qtunes.core.*;
import org.qtunes.db.*;
import java.util.*;

/**
 * A read/write {@link Store} which is used as temporary storage, before being
 * merged with a more permanent Store and saved to disk
 */
class MemoryStore implements Store {

    public static final Map<Field,Object> DEL = Collections.unmodifiableMap(new HashMap<Field,Object>());
    private List<Map<Field,Object>> fields = new ArrayList<Map<Field,Object>>();
    private BitSet indices = new BitSet();
    private int revision;

    public boolean isValid() {
        return true;
    }

    public BitSet getTracks() {
        return indices;
    }

    public int[] getTracks(String sort, String filter) {
        throw new UnsupportedOperationException();
    }

    public void set(int track, Field field, Object data) {
        while (track >= fields.size()) {
            fields.add(null);
        }
        Map<Field,Object> map = fields.get(track);
        if (map==DEL) {
            throw new IllegalStateException("Track "+track+" has been deleted");
        } if (map==null) {
            map = new HashMap<Field,Object>();
            fields.set(track, map);
            indices.set(track);
        }
        revision++;
        map.put(field, data);
    }

    public Object get(int track, Field field) {
        Map map = null;
        if (track < fields.size()) {
            map = (Map)fields.get(track);
        }
        if (map!=null) {
            return map.get(field);
        }
        return null;
    }

    public boolean isDeleted(int track) {
        return fields.size() > track && fields.get(track) == DEL;
    }

    public void delete(int track) {
        revision++;
        while (track >= fields.size()) {
            fields.add(null);
        }
        indices.clear(track);
        fields.set(track, DEL);
    }

    public BitSet getFields(int track) {
        Map map = null;
        if (track < fields.size()) {
            map = (Map)fields.get(track);
        }
        BitSet s = new BitSet();
        if (map!=null) {
            for (int i=0;i<Field.ALLFIELDS.length;i++) {
                if (map.containsKey(Field.ALLFIELDS[i])) {
                    s.set(i);
                }
            }
        }
        return s;
    }

    public int getRevision() {
        return revision;
    }

    public void merge(Store store) {
        throw new UnsupportedOperationException();
    }

    public void setFile(String key) {
    }

}
