package org.qtunes.db.spi.simple;

import java.text.*;
import java.util.*;

/**
 * A String Comparator which ignores accents on letters when sorting, and which
 * optionally strings the word "the" from the prefix and orders numbers after letters
 */
class NormalizingComparator implements Comparator<String> {

    private static final String SEARCHTOKENS = "abcdefghijklmnopqrstuvwxyz0123456789 ";
    private final boolean stripthe, numberslast;

    NormalizingComparator(boolean stripthe, boolean numberslast) {
        this.stripthe = stripthe;
        this.numberslast = numberslast;

    }

    /**
     * Given a String, remove all the punctuation and diacritics and return it
     */
    public static String clean(String s) {
        s = Normalizer.normalize(s.toLowerCase(), Normalizer.Form.NFD);
        StringBuilder b = new StringBuilder();
        for (int i=0;i<s.length();i++) {
            char c = s.charAt(i);
            if (SEARCHTOKENS.indexOf(c) >= 0) {
                b.append(c);
            }
        }
        return b.toString();
    }

    public int compare(String s1, String s2) {
        s1 = Normalizer.normalize(s1, Normalizer.Form.NFKD);
        s2 = Normalizer.normalize(s2, Normalizer.Form.NFKD);
        s1 = s1.trim().toLowerCase();
        s2 = s2.trim().toLowerCase();
        int i=0, j=0;
        char c1 = 0, c2 = 0;
        if (stripthe) {
            if (s1.startsWith("the ")) s1 = s1.substring(4);
            if (s2.startsWith("the ")) s2 = s2.substring(4);
        }
        while (i < s1.length() && j < s2.length()) {
            if (!Character.isLetterOrDigit(c1=s1.charAt(i))) {
                c1 = 0;
                i++;
            } else if (!Character.isLetterOrDigit(c2=s2.charAt(j))) {
                c2 = 0;
                j++;
            } else if (c1 < c2) {
                return numberslast && c1 <= '9' && c2 > '9' ? 1 : -1;
            } else if (c2 < c1) {
                return numberslast && c2 <= '9' && c1 > '9' ? -1 : 1;
            } else {
                i++;
                j++;
            }
        }
        if (i==s1.length()) {
            if (j==s2.length()) {
                return 0;               // even
            } else if (c1==0) {
                return 1;               // len(s1)=0, len(s2)>0         s2
            } else {
                return -1;              // len(s2)>len(s1)               s2
            }
        } else if (c2==0) {             // len(s1)>0, len(s2)=0         s1
            return -1;
        } else {                        // len(s1)>len(s2)              s2
            return 1;
        }
    }

    public static void main(String[] args) throws Exception {
        NormalizingComparator c = new NormalizingComparator(true, true);
        Arrays.sort(args, c);
        for (int i=0;i<args.length;i++) {
            System.out.println("\""+args[i]+"\"");
        }
    }

}
