package org.qtunes.ff;

import org.qtunes.core.util.*;
import org.qtunes.core.*;
import org.qtunes.db.*;
import java.util.*;
import java.io.*;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.UnsupportedAudioFileException;

/**
 * A TrackReader loads the tracks from disk
 */
public abstract class TrackReader implements Service {

    public static final String[] MP3GENRES = {
        "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", "Grunge", "Hip-Hop",
        "Jazz", "Metal", "New Age", "Oldies", "Other", "Pop", "R&B", "Rap", "Reggae",
        "Rock", "Techno", "Industrial", "Alternative", "Ska", "Death Metal", "Pranks",
        "Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk", "Fusion",
        "Trance", "Classical", "Instrumental", "Acid", "House", "Game", "Sound Clip", "Gospel",
        "Noise", "AlternRock", "Bass", "Soul", "Punk", "Space", "Meditative", "Instrumental Pop",
        "Instrumental Rock", "Ethnic", "Gothic", "Darkwave", "Techno-Industrial", "Electronic",
        "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy", "Cult", "Gangsta", "Top 40",
        "Christian Rap", "Pop/Funk", "Jungle", "Native American", "Cabaret", "New Wave",
        "Psychadelic", "Rave", "Showtunes", "Trailer", "Lo-Fi", "Tribal", "Acid Punk",
        "Acid Jazz", "Polka", "Retro", "Musical", "Rock & Roll", "Hard Rock", "Folk",
        "Folk-Rock", "National Folk", "Swing", "Fast Fusion", "Bebob", "Latin", "Revival",
        "Celtic", "Bluegrass", "Avantgarde", "Gothic Rock", "Progressive Rock",
        "Psychedelic Rock", "Symphonic Rock", "Slow Rock", "Big Band", "Chorus", "Easy Listening",
        "Acoustic", "Humour", "Speech", "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony",
        "Booty Bass", "Primus", "Porn Groove", "Satire", "Slow Jam", "Club", "Tango", "Samba",
        "Folklore", "Ballad", "Power Ballad", "Rhythmic Soul", "Freestyle", "Duet", "Punk Rock",
        "Drum Solo", "A capella", "Euro-House", "Dance Hall"
     };

    /**
     * Return the {@link AudioInputStream} for the specified File. We
     * use this approach because it's more reliable than relying on
     * the {@link javax.sound.sampled.AudioSystem} to correctly identify
     * the file.
     */
    public abstract AudioInputStream getAudioInputStream(File file) throws UnsupportedAudioFileException, IOException;

    /**
     * Get the type of track read by this reader, eg "mp3". Must match the
     * "Type" field of the FieldMap read by this reader
     */
    public abstract String getType();

    /**
     * Return true if this TrackReader matches the File
     */
    public abstract boolean matches(File file);

    /**
     * Load and parse the file and return a {@link FieldMap} with its fields
     */
    public abstract FieldMap read(File file) throws IOException;

    /**
     * Return the appropriate TracKReader for the specified type
     */
    public static final TrackReader getTrackReader(ServiceContext context, String type) {
        TrackReader[] readers = context.getServices(TrackReader.class, null);
        for (int i=0;i<readers.length;i++) {
            if (readers[i].getType().equals(type)) {
                return readers[i];
            }
        }
        return null;
    }

    /**
     * Return the appropriate TracKReader for the specified file
     */
    public static final TrackReader getTrackReader(ServiceContext context, File file) throws IOException {
        TrackReader[] readers = context.getServices(TrackReader.class, null);
        file = FileUtils.getAbsoluteFile(file);
        for (int i=0;i<readers.length;i++) {
            if (readers[i].matches(file)) {
                return readers[i];
            }
        }
        return null;
    }

    private static TrackReader[] allreaders;

    private synchronized static void initglobalreaders() {
        if (allreaders == null) {
            allreaders = new TrackReader[] {
                new org.qtunes.ff.spi.mp3.MP3Handler(),
                new org.qtunes.ff.spi.mp4.MP4Handler() 
            };
        }
    }

    /**
     * Return the appropriate TracKReader for the specified file - the list
     * of readers is built in and no context is required
     */
    public static final TrackReader getTrackReader(File file) throws IOException {
        initglobalreaders();
        file = FileUtils.getAbsoluteFile(file);
        for (int i=0;i<allreaders.length;i++) {
            if (allreaders[i].matches(file)) {
                return allreaders[i];
            }
        }
        return null;
    }

    public static final TrackReader getTrackReader(String type) {
        initglobalreaders();
        for (int i=0;i<allreaders.length;i++) {
            if (allreaders[i].getType().equals(type)) {
                return allreaders[i];
            }
        }
        return null;
    }

}
