package org.qtunes.ff.spi.mp3;

import java.nio.*;
import java.io.*;

class TagImage extends Tag {

    private String mimetype, description;
    private int length, picturetype;
    private long offset;

    TagImage() {
    }

    private synchronized void parse() throws IOException {
        if (offset==0) {
            try {
                ByteBuffer data = getContent();
                data.rewind();
                byte encoding = data.get();
                if (getVersion()==2) {
                    mimetype = readFixedString(data, 3);
                } else {
                    mimetype = readEncodedString(data, (byte)0);
                }
                picturetype = data.get() & 0xFF;
                description = readEncodedString(data, encoding);
                length = data.remaining();

                if (mimetype.equals("PNG")) {
                    mimetype = "image/png";
                } else if (mimetype.equals("JPG")) {
                    mimetype = "image/jpeg";
                } else if (mimetype.equals("GIF")) {
                    mimetype = "image/gif";
                } else if (mimetype.equals("image/jpg")) {
                    mimetype = "image/jpeg";
                }
                offset = getFileOffset() + getContentOffset() + data.position();
            } catch (BufferUnderflowException e) {
                throw new IOException(e);
            }
        }
    }

    /**
     * Get the offset of the image content in the original stream
     */
    long getOffset() throws IOException {
        parse();
        return offset;
    }

    /**
     * Get the length of the image content in the original stream
     */
    int getLength() throws IOException {
        parse();
        return length;
    }

    /**
     * Get the "picture type" of the image as defined in iD3 v 2.4
     */
    int getType() throws IOException {
        parse();
        return picturetype;
    }

    /**
     * Get the MIME-type of the image
     */
    String getMIMEType() throws IOException {
        parse();
        return mimetype;
    }

    /**
     * Get the "description" of the image, as described in ID3 v2.4
     */
    String getDescription() throws IOException {
        parse();
        return description;
    }

    public String toString() {
        try {
            return "["+getName()+" type="+getType()+" mime="+getMIMEType()+" desc="+getDescription()+" pos="+getOffset()+"/"+getLength()+"]";
        } catch (IOException e) {
            return super.toString();
        }
    }

}
