package org.qtunes.ff.spi.mp3;

import java.io.*;
import java.nio.*;

class TagNumber extends Tag {

    private boolean read, invalid;
    private long value;

    TagNumber() {
    }

    TagNumber(String name, int version, int flags, int value) {
        super(name, version, flags, encode(version, value));
    }

    private static byte[] encode(int version, int value) {
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            out.write(0);
            writeString(0, Integer.toString(value), out);
            return out.toByteArray();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private synchronized void parse() throws IOException {
        if (!read) {
            try {
                ByteBuffer data = getContent();
                data.rewind();
                byte encoding = data.get();
                String s = readEncodedString(data, encoding);
                try {
                    value = Long.parseLong(s);
                } catch (NumberFormatException e) {
                    invalid = true;
                    throw new IOException("Invalid number \""+s+"\"");
                }
                read = true;
            } catch (BufferUnderflowException e) {
                invalid = true;
                throw new IOException("Invalid number (empty)");
            }
        }
    }

    long getValue() throws IOException {
        parse();
        return value;
    }

    public String toString() {
        try {
            return "["+getName()+" value="+getValue()+"]";
        } catch (IOException e) {
            return super.toString();
        }
    }

    boolean isInvalid() {
        return invalid;
    }

}
