package org.qtunes.ff.spi.mp3;

import java.io.*;
import java.nio.*;

/**
 * A Position tag is used to store track/disc numbers as "1" or "2/12"
 */
class TagPosition extends Tag {

    private boolean read, invalid;
    private int pos, range;

    TagPosition() {
    }

    TagPosition(String tag, int version, int flags, int position, int range) {
        super(tag, version, flags, TagText.encode(version, position+"/"+range));
    }

    private synchronized void parse() throws IOException {
        if (!read) {
            try {
                ByteBuffer data = getContent();
                data.rewind();
                byte encoding = data.get();
                String s = readEncodedString(data, encoding);
                int off = s.indexOf('/');
                try {
                    if (off < 0) {
                        pos = Integer.parseInt(s);
                    } else {
                        pos = Integer.parseInt(s.substring(0, off));
                        range = Integer.parseInt(s.substring(off+1));
                        if (range < pos) {
                            range = 0;
                        }
                    }
                } catch (Exception e) {
                    invalid = true;
                    throw new IOException("Invalid position \""+s+"\"");
                }
                read = true;
            } catch (BufferUnderflowException e) {
                throw new IOException(e);
            }
        }
    }

    boolean isInvalid() {
        return invalid;
    }

    int getPosition() throws IOException {
        parse();
        return pos;
    }

    int getRange() throws IOException {
        parse();
        return range;
    }

    public String toString() {
        try {
            return "["+getName()+" value="+getPosition()+"/"+getRange()+"]";
        } catch (IOException e) {
            return super.toString();
        }
    }

}
