package org.qtunes.ff.spi.mp3;

import java.io.*;
import java.nio.*;

class TagText extends Tag {

    private String value;

    TagText() {
    }

    TagText(String name, int version, int flags, String value) {
        super(name, version, flags, encode(version, value));
    }

    protected TagText(String name, int version, int flags, byte[] value) {
        super(name, version, flags, value);
    }

    static byte[] encode(int version, String value) {
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            int encoding = getStringEncoding(version, value);
            out.write(encoding);
            writeString(encoding, value, out);
            return out.toByteArray();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private synchronized void parse() throws IOException {
        if (value==null) {
            try {
                ByteBuffer data = getContent();
                data.rewind();
                byte encoding = data.remaining() > 0 ? data.get() : 0;
                value = readEncodedString(data, encoding);
            } catch (BufferUnderflowException e) {
                throw new IOException(e);
            }
        }
    }

    String getValue() throws IOException {
        parse();
        return value;
    }

    public String toString() {
        try {
            return "["+getName()+" text="+getValue()+"]";
        } catch (IOException e) {
            return super.toString();
        }
    }

    boolean isInvalid() {
        ByteBuffer data = getContent();
        data.rewind();
        byte encoding = data.remaining() > 0 ? data.get() : 0;
        if (encoding == 0) {
            try {
                getValue();
            } catch (IOException e) {
                return true;
            }
            if (value != null) {
                for (int i=0;i<value.length();i++) {
                    char c = value.charAt(i);
                    if (c > 127 || c < 32) return true;
                }
            }
        }
        return false;
    }

}
