package org.qtunes.web.spi.simple;

import org.qtunes.core.*;
import org.qtunes.web.*;
import java.net.*;
import java.io.*;
import java.util.*;

public class SimpleServer implements WebService, Runnable {

    private ServiceContext context;
    private Thread main;
    private volatile boolean cancelled;
    private Map<String,WebHandler> handlers = new HashMap<String,WebHandler>();
    private ServerSocket serversocket;

    private final WebHandler notfound = new WebHandler() {
        public void handle(WebConnection con) throws IOException {
            con.sendResponseHeaders(404, -1);
        }
    };

    public void run() {
        while (!cancelled) {
            try {
                final Socket clientsocket = serversocket.accept();
                Thread thread = new Thread() {
                    public void run() {
                        SimpleConnection con = null;
                        try {
                            con = new SimpleConnection(clientsocket.getInputStream(), clientsocket.getOutputStream(), (InetSocketAddress)clientsocket.getRemoteSocketAddress(), SimpleServer.this);
                            while (con.parse()) {
                                getHandler(con.getRequestURI().getPath()).handle(con);
                            }
                        } catch (IOException e) {
                            context.warn(null, e);
                            if (con.getResponseCode() < 0) {
                                try {
                                    con.sendResponseHeaders(400, -1);
                                } catch (IOException e2) {}
                            }
                        } finally {
                            if (con!=null) {
                                try { con.destroy(); } catch (IOException e) { context.warn(null, e); }
                            }
                        }
                    }
                };
                thread.start();
            } catch (IOException e) {
                if (!cancelled) {
                    context.warn(null, e);
                }
            }
        }
    }

    public InetSocketAddress getAddress() {
        return (InetSocketAddress)serversocket.getLocalSocketAddress();
    }

    public String hasHandler(String path) {
        for (Iterator<Map.Entry<String,WebHandler>> i = handlers.entrySet().iterator();i.hasNext();) {
            Map.Entry<String,WebHandler> e = i.next();
            if (path.startsWith(e.getKey())) {
                return e.getKey();
            }
        }
        return null;
    }

    public void setHandler(String path, WebHandler handler) {
        handlers.put(path, handler);
    }

    private WebHandler getHandler(String path) {
        String lastkey = "";
        WebHandler lasthandler = null;
        for (Iterator<Map.Entry<String,WebHandler>> i = handlers.entrySet().iterator();i.hasNext();) {
            Map.Entry<String,WebHandler> e = i.next();
            if (path.startsWith(e.getKey()) && e.getKey().length() > lastkey.length()) {
                lastkey = e.getKey();
                lasthandler = e.getValue();
            }
        }
        return lasthandler == null ? notfound : lasthandler;
    }

    //---------------------------------------------------------------------------

    public ServiceContext getContext() {
        return context;
    }

    public synchronized void startService(ServiceContext context) {
        this.context = context;
        int port = Integer.parseInt(context.getProperty("port"));
        try {
            serversocket = new ServerSocket(port);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        main = new Thread(this);
        main.start();
    }

    public synchronized void stopService(ServiceContext context) {
        cancelled = true;
        if (main!=null) {
            try {
                serversocket.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            main = null;
            serversocket = null;
        }
    }

    public Map<String,Object> reportState() {
        Map<String,Object> map = new LinkedHashMap<String,Object>();
        map.put("port", getAddress().getPort());
        return map;
    }

}
