package org.qtunes.web.spi.sun;

import org.qtunes.web.*;
import java.util.*;
import java.text.*;
import java.io.*;
import java.net.*;
import com.sun.net.httpserver.*;
import org.qtunes.core.util.HttpParameters;

class SunConnection implements WebConnection {

    private HttpParameters parameters;
    private HttpExchange exchange;
    private String path;
    private static final String encoding = "ISO-8859-1";
    private final WebService webservice;

    SunConnection(HttpExchange exchange, WebService webservice) {
        this.exchange = exchange;
        this.webservice = webservice;
    }

    public WebService getWebService() {
        return webservice;
    }

    public InetSocketAddress getRemoteAddress() {
        return exchange.getRemoteAddress();
    }

    public URI getRequestURI() {
        return exchange.getRequestURI();
    }

    public int getResponseCode() {
        return exchange.getResponseCode();
    }

    public void setPath(String path) {
        this.path = path;
    }

    public void close() {
        exchange.close();
    }

    public String getPath() {
        return path;
    }

    public void sendResponseHeaders(int status, int length) throws IOException {
        exchange.sendResponseHeaders(status, length);
    }

    public OutputStream getOutputStream() {
        return exchange.getResponseBody();
    }

    public InputStream getInputStream() {
        return exchange.getRequestBody();
    }

    public String getMethod() {
        return exchange.getRequestMethod();
    }

    public String getRequestHeader(String key) {
        return exchange.getRequestHeaders().getFirst(key);
    }

    public List<String> getRequestHeaders(String key) {
        return exchange.getRequestHeaders().get(key);
    }

    public Collection<String> getRequestHeaderNames() {
        return exchange.getRequestHeaders().keySet();
    }

    public void setResponseHeader(String key, String value) {
        if (value==null) {
            exchange.getResponseHeaders().remove(key);
        } else {
            exchange.getResponseHeaders().set(key, value);
        }
    }

    public void setResponseHeader(String key, Date value) {
        setResponseHeader(key, new SimpleDateFormat(WebHandler.DATEFORMAT).format(value));
    }

    public void addResponseHeader(String key, String value) {
        exchange.getResponseHeaders().add(key, value);
    }


    private synchronized HttpParameters doGetParameters() {
        if (parameters==null) {
            try {
                parameters = new HttpParameters(exchange.getRequestURI().getRawQuery(), encoding);
            } catch (UnsupportedEncodingException e) {
                throw new RuntimeException(e);
            }
        }
        return parameters;
    }

    public String getParameter(String key) {
        return doGetParameters().getValue(key);
    }

    public List<String> getParameterValues(String key) {
        return doGetParameters().getValues(key);
    }

    public Map<String,Object> getParameters() {
        return doGetParameters().getAllValues();
    }

}
