package org.qtunes.webplayer;

import org.qtunes.core.*;
import org.qtunes.auth.*;
import org.qtunes.player.*;
import org.qtunes.speaker.*;
import org.qtunes.db.*;
import org.qtunes.auth.control.*;
import org.qtunes.web.*;
import org.qtunes.core.util.JSONSerializer;
import org.qtunes.auth.control.ControlJSONSerializer;
import java.net.*;
import java.io.*;
import java.util.*;

class StatusHandler implements WebHandler, Listener {
   
    private final ServiceContext context;
    private final Database db;
    private final Auth auth;
    private JSONSerializer json;
    private int currentupdate;
    private int timeout = 10000;
    private Exception exception;

    StatusHandler(WebPlayer player) {
        this.context = player.getContext();
        this.db = context.getService(Database.class);
        this.auth = context.getService(Auth.class);
        this.json = new ControlJSONSerializer();
        this.currentupdate = 1;
    }

    public void handle(WebConnection con) throws IOException {
        try {
            int update = Integer.parseInt(con.getParameter("update"));
            int session = Integer.parseInt(con.getParameter("session"));
            Session.setCurrentSession(auth.getSession(session));
            Session.getCurrentSession().touch();
            waitForUpdate(update);
            con.setResponseHeader("Server", context.getSoftwareName()+"/"+con.getWebService().getContext().getServiceName());
            con.setResponseHeader("Date", new Date());
            con.addResponseHeader("Content-Type", "application/json;charset=UTF-8");
            String response = json.write(getStatus());
//            System.out.println("STATUS="+response);
            byte[] data = response.getBytes("UTF-8");
            con.sendResponseHeaders(200, data.length);
            OutputStream out = con.getOutputStream();
            out.write(data);
            out.close();
        } finally {
            Session.setCurrentSession(Session.NONE);
        }
    }

    public synchronized void handleEvent(String name, java.util.Map<?,?> properties) {
        if (name.equals("error") && properties.get("service") instanceof Player) {
            exception = (Exception)properties.get("exception");
        }
        currentupdate++;
        notifyAll();
    }

    private void waitForUpdate(int update) {
        if (update >= currentupdate) {
            synchronized(this) {
                try {
                    wait(timeout);
                } catch (InterruptedException e) {}
            }
        }
    }

    private Map getStatus() {
        Map<String,Object> map = new LinkedHashMap<String,Object>();
        map.put("name", context.getServerName());
        map.put("update", currentupdate);
        Map<String,Object> players = new LinkedHashMap<String,Object>();
        Map<String,Object> speakers = new LinkedHashMap<String,Object>();
        String[] servicenames = context.getServiceNames();
        for (int i=0;i<servicenames.length;i++) {
            String name = servicenames[i];
            Service service = context.getService(Service.class, "name=\""+name+"\"");
            if (service!=null && service.getContext().isActive()) {
                Map<String,Object> state = service.reportState();
                if (state!=null) {
                    if (service instanceof Player) {
                        players.put(name, state);
                        map.put("players", players);
                    } else if (service instanceof Speaker) {
                        speakers.put(name, state);
                        map.put("speakers", speakers);
                    } else {
                        map.put(name, state);
                    }
                }
            }
        }
        if (exception!=null) {
            map.put("error", exception);
            exception = null;
        }
        return map;
    }

}
