package org.qtunes.webplayer;

import org.qtunes.core.*;
import org.qtunes.auth.*;
import org.qtunes.player.*;
import org.qtunes.db.*;
import org.qtunes.zeroconf.*;
import org.qtunes.auth.control.*;
import org.qtunes.web.*;
import org.qtunes.core.util.Base64;
import org.qtunes.core.util.JSONSerializer;
import org.qtunes.core.util.HttpParameters;
import java.net.*;
import java.io.*;
import java.util.*;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

public class WebPlayer implements Service {
   
    private ZCService serviceinfo;
    private ServiceContext context;
    private WebAuthorizer authorizer;
    private WebService webservice;
    private String gridprefix = "/webplayer/grid";
    private String statusprefix = "/webplayer/status";
    private String staticprefix = "/webplayer/static/";
    private String controlprefix = "/webplayer/control/";
    private String imageprefix = "/webplayer/image";
    private Database db;
    private StaticWebHandler statichandler, iconhandler;
    private WebHandler gridhandler, imagehandler;
    private StatusHandler statushandler;
    private WebController controlhandler;

    public void startService(ServiceContext context) {
        this.context = context;
        this.authorizer = new WebAuthorizer(this);
        String port = context.getProperty("port");
        this.webservice = context.getService(WebService.class, port==null ? null : "port="+port);
        this.statichandler = new StaticWebHandler(context, staticprefix, "html/", getClass());
        this.iconhandler = new StaticWebHandler(context, "/favicon.ico", "html/favicon.png", getClass());
        this.gridhandler = new CachingWebHandler(new GridHandler(this));
        this.statushandler = new StatusHandler(this);
        this.imagehandler = new ImageHandler(this);
        this.controlhandler = new WebController(this, controlprefix, authorizer);
        webservice.setHandler(statichandler.getPrefix(), statichandler);
        webservice.setHandler(iconhandler.getPrefix(), iconhandler);
        webservice.setHandler(gridprefix, gridhandler);
        webservice.setHandler(statusprefix, statushandler);
        webservice.setHandler(controlprefix, controlhandler);
        webservice.setHandler(imageprefix, imagehandler);
        context.addListener(statushandler);
        context.getService(Auth.class).addAuthorizer(authorizer);

        ZeroConf zeroconf = context.getService(ZeroConf.class);
        Map<String,String> props = new HashMap<String,String>();
        props.put("path", staticprefix + "index.html");
        if (zeroconf != null) {
            serviceinfo = zeroconf.register("_http._tcp", context.getServerName(), webservice.getAddress().getPort(), props);
        }
    }

    public void stopService(ServiceContext context) {
        ZeroConf zeroconf = context.getService(ZeroConf.class);
        if (zeroconf != null) {
            zeroconf.unregister(serviceinfo);
        }
        context.removeListener(statushandler);
        webservice.setHandler(gridprefix, null);
        webservice.setHandler(statusprefix, null);
        webservice.setHandler(controlprefix, null);
        webservice.setHandler(imageprefix, null);
        webservice.setHandler(statichandler.getPrefix(), null);
        webservice.setHandler(iconhandler.getPrefix(), null);
        context.getService(Auth.class).removeAuthorizer(authorizer);
    }

    public ServiceContext getContext() {
        return context;
    }

    public Map<String,Object> reportState() {
        return null;
    }
}
