Ext.ns("QTunes");
QTunes.PlayPanel = Ext.extend(Ext.Panel, (function() {

var qtunes;
var players = new Array();
var player;
var playerUpdateListener;
var holdstate = 0;

var formatTime = function(ms) {
    var secs = Math.floor(ms / 1000);
    var hours = Math.floor(secs / 3600);
    var mins = Math.floor((secs / 60) % 60);
    secs %= 60;
    if (secs < 10) secs = "0"+secs;
    var out;
    if (hours) {
        if (mins < 10) mins = "0"+mins;
        out = hours+":"+mins+":"+secs;
    } else {
        out = mins+":"+secs;
    }
    return out;
};

var updatePlayPauseIcon = function() {
    var cmp = Ext.getCmp("playPauseButton");
    if (holdstate == 1) {
        cmp.setIcon("btn-stop.png");
    } else if (player && player.isPlaying()) {
        cmp.setIcon("btn-pause.png");
    } else {
        cmp.setIcon("btn-play.png");
    }
};

return {
    constructor: function(config) {
        qtunes = config.qtunes;
        var that = this;

        var x1 = 310;
        var x2 = 0;
        var yval = 5;
        var heightval = 30;
        config = Ext.apply({
            id: "playpanel",
            width: 600,
            height:50, 
            layout: 'absolute',
            items:[
            {
                xtype: 'panel',
                x: x1,
                y: yval,
                scale:'medium',
                xtype: 'button',
                cls: "playbutton",
                icon: "btn-prev.png",
                id: 'prevButton',
                disabled: true,
                listeners: {
                    click: function(item) {
                        that.playPrev();
                    }
                }
            },
            {
                xtype: 'button',
                x: x1 + 30,
                y: yval,
                scale: 'medium',
                icon: "btn-pause.png",
                cls: "playbutton",
                id: 'playPauseButton',
                disabled: true,
                listeners: {
                    click: function(item) {
                        that.playPause();
                    },
                    render: function(c) {
                        c.getEl().on({
                            mousedown : function() {
                                that.buttonchangetask = setTimeout(function() {
                                    holdstate = 1;
                                    updatePlayPauseIcon();
                                }, 1000);
                            },
                            mouseout: function() {
				// Would have liked to enable move out then in
				// with mouse button still depressed and have
				// holdstate maintained, however this involved
				// identifying either a) mouseup when cursor
				// is on another target, or b) mouseover when
				// button is depressed. Can't check button status
				// for (b) (not part of mouseover/mousemove
				// events) and too complex to register handlers
				// on everything else for (a). So move off while
				// button depressed clears everything.
                                clearTimeout(that.buttonchangetask);
				holdstate = 0;
                                updatePlayPauseIcon();
                            },
                            mouseup: function() {
                                clearTimeout(that.buttonchangetask);
                            },
                        });
                    },
                }
            },
            {
                xtype: 'button',
                x: x1 + 60,
                y: yval,
                scale: 'medium',
                icon: "btn-next.png",
                cls: "playbutton",
                id: 'nextButton',
                disabled: true,
                listeners: {
                    click: function(item) {
                        that.playNext();
                    }
                }
            },
            {
                xtype: 'slider',
                id: 'volumeSlider',
                x: x1 + 5,
                y: 30,
                width: 115,
                height:15,
                minvalue: 0,
                maxvalue: 100,
                listeners: {
                    changecomplete: function(item, value, thumb) {
                        that.setVolume(value);
                    }
                }
            },
            {
                xtype: "textfield",
                x: x1 + 100,
                y: yval + 4,
                width: 100,
                id: "searchField",
                blankText: "Search",
                selectOnFocus: true,
                enableKeyEvents: true,
                listeners: {
                    focus: function(item, event) {
                        this.origvalue = item.getValue();
                    },
                    keyup: function(item, event) {
                        that.updateSearch(item.getValue());
                    },
                    specialkey: function(item, event) {
                        if (event.getKey() == event.ENTER) {
                            that.search(item.getValue());
                            that.findById("volumeSlider").focus();
                        } else if (event.getKey() == event.ESCAPE) {
                            item.setValue(this.origvalue);
                            that.findById("volumeSlider").focus();
                        }
                    }
                }
            },
            {
                xtype: 'button',
                x: x1 + 130,
                y: 30,
                scale: 'small',
                icon: "btn-shuffleoff.png",
                id: 'shuffleButton',
                listeners: {
                    click: function(item) {
                        that.toggleShuffle();
                    }
                }
            },
            {
                xtype: 'button',
                x: x1 + 155,
                y: 30,
                scale: 'small',
                icon: "btn-repeatoff.png",
                id: 'repeatButton',
                listeners: {
                    click: function(item) {
                        that.toggleRepeat();
                    }
                }
            }, 
            {
                xtype: 'button',
                x: x1 + 180,
                y: 30,
                scale: 'small',
                icon: "btn-speaker.png",
                id: 'speakerButton',
                listeners: {
                    click: function(item) {
                        that.speakerSetup();
                    }
                }
            },
            {
                xtype: 'panel',
                id: 'nowPlayingPanel',
                layout: 'absolute',
                x: x2,
                y: 0,
                width:301,
                height: 50,
                items: [{
                    x: 0,
                    y: 5,
                    width:300,
                    height:30,
                    align: 'center',
                    id: 'trackDetail',
                },{
                    x: 0,
                    y: 33,
                    width:50,
                    height:20,
                    region:'west',
                    id: 'trackElapsed',
                    text: '0:00',
                    xtype: 'label',
                },{
                    x: 50,
                    y:29,
                    width:200,
                    height:20,
                    xtype: 'slider',
                    id: 'positionSlider',
                    minvalue: 0,
                    maxvalue: 0,
                    listeners: {
                        changecomplete: function(item, value, thumb) {
                            that.seek(value);
                        }
                    }
                },{
                    x: 250,
                    y: 33,
                    width:50,
                    height:20,
                    region:'east',
                    id: 'trackRemaining',
                    xtype: 'label',
                    text: '0:00',
                }]
            }]
        }, config);
        QTunes.PlayPanel.superclass.constructor.call(this, config);

        playerUpdateListener = function() {
            var isactive = player.getPlaylistIndex() >= 0;

            updatePlayPauseIcon();
            Ext.getCmp("shuffleButton").setIcon(player.getShuffle() ? "btn-shuffleon.png" : "btn-shuffleoff.png");
            Ext.getCmp("repeatButton").setIcon(player.getRepeat()==0 ? "btn-repeatoff.png" : player.getRepeat()==1 ? "btn-repeatone.png" : "btn-repeatall.png");
            Ext.getCmp("volumeSlider").setValue(player.getVolume());
            Ext.getCmp("nextButton").setDisabled(!isactive);
            Ext.getCmp("playPauseButton").setDisabled(player.getPlaylistLength() == 0);
            Ext.getCmp("prevButton").setDisabled(!isactive);
            var speakers = player.getSpeakers();
            var hasgain = false;
            for (var x=0;speakers && x<speakers.length;x++) {
                hasgain |= qtunes.getSpeaker(speakers[x]).hasGain();
            }
            Ext.getCmp("volumeSlider").setDisabled(!hasgain);

            if (isactive) {
                that.ms = player.getMs();
                that.duration = player.getCurrentDuration();
                that.isplaying = !player.isPaused();
                var text = "<b>"+player.getCurrentTitle()+"</b><br/>"+player.getCurrentArtist();
                that.findById("trackDetail").body.update(text);
                Ext.getCmp("artpanel").setTrack(player.getCurrentIndex());
            } else {
                that.ms = that.duration = 0;
                that.isplaying = false;
                that.findById("trackDetail").body.update("");
            }
        };
    },

    addPlayer: function(newplayer) {
        qtunes.debug("playPanel.addPlayer("+newplayer+")");
        players[players.length] = newplayer;
        if (players.length == 1) {
            this.setPlayer(newplayer);
            var that = this;
            this.ticker = {
                run: function() {
                    var elapsedtext = formatTime(that.ms);
                    var remainingtext = formatTime(that.duration);
                    that.findById("trackElapsed").setText(elapsedtext);
                    that.findById("trackRemaining").setText(remainingtext);
                    that.findById("positionSlider").setMaxValue(that.duration);
                    that.findById("positionSlider").setValue(that.ms, false);
                    if (that.isplaying) {
                        that.ms += 1000;
                        if (that.ms >= that.duration) {
                            that.isplaying = 0;
                            that.ms = that.duration = 0;
                        }
                    }
                },
                interval: 1000,
            };
            Ext.TaskMgr.start(this.ticker);
        }
    },

    removePlayer: function(newplayer) {
        qtunes.debug("playPanel.removePlayer("+newplayer+")");
        if (player == newplayer) {
            this.setNextPlayer();
        }
        for (var i=0;i<players.length;i++) {
            if (players[i] == newplayer) {
                players[i].splice(i, 1);
            }
        }
    },

    getPlayer: function() {
        return player;
    },

    setNextPlayer: function() {
        qtunes.debug("playPanel.setNextPlayer()");
        for (var i=0;i<players.length;i++) {
            if (players[i] == player) {
                i++;
                if (i==players.length) {
                    i = 0;
                }
                this.setPlayer(players[i]);
                break;
            }
        }
    },

    setPlayer: function(newplayer) {
        if (player != newplayer) {
            if (player) {
                player.un("update", playerUpdateListener);
            }
            qtunes.debug("playPanel.setPlayer("+newplayer+")");
            player = newplayer;
            player.on("update", playerUpdateListener);
            console.log("Setting to "+player);
            qtunes.setPlayer(player);
            console.log("Set to "+qtunes.getPlayer());
            playerUpdateListener();
        }
    },

    playNext: function() {
        qtunes.debug("playPanel.playNext()");
        qtunes.actionPlayNext(player);
    },

    playPrev: function() {
        qtunes.debug("playPanel.playPrev()");
        if (this.ms < 5000) {
            qtunes.actionPlayPrevious(player);
        } else {
            qtunes.actionSeek(player, 0);
        }
    },

    playPause: function() {
        qtunes.debug("playPanel.playPause() hs="+holdstate);
        if (holdstate == 1) {
            qtunes.actionClear(player);
            holdstate = 0;
            updatePlayPauseIcon();
        } else {
            qtunes.actionSetPaused(player, !player.isPaused());
        }
    },

    toggleShuffle: function() {
        qtunes.debug("playPanel.toggleShuffle()");
        qtunes.actionSetShuffle(player, !player.getShuffle());
    },

    toggleRepeat: function() {
        qtunes.debug("playPanel.toggleRepeat()");
        qtunes.actionSetRepeat(player, (player.getRepeat()+1)%3);
    },

    speakerSetup: function() {
        qtunes.debug("playPanel.speakerSetup()");
        qtunes.speakerSetup();
    },

    seek: function(position) {
        qtunes.debug("playPanel.seek("+position+")");
        this.ms = position;
        qtunes.actionSeek(player, position);
    },

    setVolume: function(volume) {
        qtunes.debug("playPanel.setVolume("+volume+")");
        qtunes.actionSetVolume(player, volume);
    },

    updateSearch: function(text) {
        qtunes.debug("updateSearch "+text);
    },

    search: function(text) {
        qtunes.debug("search "+text);
        qtunes.setFilter(text);
    },


};

}()));
