Ext.ns("QTunes");
QTunes.Player = Ext.extend(Ext.util.Observable, (function() {

var qtunes;

return {

    properties: false,

    constructor: function(config) {
        qtunes = config.qtunes;
        this.addEvents({"update": true});
        this.listeners = config.listeners;

        this.getName = function() {
            return config.name;
        };

        this.getDisplay = function() {
            return config.display;
        };

        this.toString = function() {
            return "[Player "+config.name+"/"+config.display+"]";
        };

        QTunes.Player.superclass.constructor.call(this, config);

        this.update(config);
    },

    update: function(properties) {
        var oldrevision = this.getRevision();
//        qtunes.debug("Player."+this.getName()+".update("+Ext.util.JSON.encode(properties)+")");
        this.properties = properties;
        if (oldrevision != this.getRevision()) {
            this.fireEvent("update");
            qtunes.getNowPlaylist().fireEvent("update");
        }
    },

    isPlaying: function() {
        return !this.isPaused() && this.getCurrentIndex() >= 0;
    },

    isPaused: function() {
        return this.properties.paused;
    },

    getSpeakers: function() {
        return this.properties.speakers;
    },

    getVolume: function() {
        return this.properties.volume;
    },

    getRepeat: function() {
        return this.properties.repeat;
    },

    getShuffle: function() {
        return this.properties.shuffle;
    },

    getMs: function() {
        return this.properties.playoffset;
    },

    getCurrentDuration: function() {
        return this.properties.np_duration;
    },

    getCurrentTitle: function() {
        return this.properties.np_title;
    },

    getCurrentArtist: function() {
        return this.properties.np_artist;
    },

    getCurrentAlbum: function() {
        return this.properties.np_album;
    },

    getCurrentTrack: function() {
        return this.properties.np_track;
    },

    getCurrentDisc: function() {
        return this.properties.np_disc;
    },

    getPlaylistIndex: function() {
        return this.properties.playlistindex;
    },

    getPlaylistLength: function() {
        return this.properties.playlistlength;
    },

    getCurrentIndex: function() {
        return this.properties.np_index;
    },

    getRevision: function() {
        return this.properties ? this.properties.revision : -1;
    },

    validatePlay: function() {
        if (this.getSpeakers().length == 0) {
            Ext.Msg.alert("Error", "Zone \""+this.getDisplay()+"\" has no speakers");
            return false;
        } else {
            return true;
        }
    }
};

}()));
