Ext.ns("QTunes");
QTunes.QTunes = Ext.extend(Ext.util.Observable, (function() {

var qtunes;
var prefix;
var playlists;
var players = new Object();
var speakers = new Object();
var player, playlist, playlistPanel, playPanel, gridPanel, session;
var rootPlaylist, nowPlaylist;

var addPlayer = function(me, player) {
    playPanel.addPlayer(player);
};

var removePlayer = function(me, player) {
    playPanel.removePlayer(player);
};

var addPlaylist = function(me, newplaylist) {
    playlistPanel.addPlaylist(newplaylist);
};

var removePlaylist = function(me, playlist) {
    playlistPanel.removePlaylist(playlist);
};

var addSpeaker = function(me, speaker) {
};

var removeSpeaker = function(me, speaker) {
};

var qcall = function(method, params, callback, nologinonfail) {
    if (session != null) {
        params.session = session.id;
    }

    var url = "";
    for (var i in params) {
        url += "&"+escape(i)+"="+escape(params[i]);
    }
    url = prefix + "control/" + method + url.replace(/^&/, "?");
//    qtunes.debug("qtunes.call: "+url);
    Ext.Ajax.request({
        url: url,
        success: function(r) {
//            qtunes.debug("qtunes.call: "+r.responseText);
            var res = Ext.util.JSON.decode(r.responseText);
            var msg, i;
            if (res.ok) {
                if (callback) {
                    callback(res.response);
                }
            } else if (!res.sesssionok && !nologinonfail) {
                qtunes.login();
                qcall(method, params, callback, true);
            } else {
                qtunes.displayThrowable(res.exception);
            }
        }
    });
};

return {
    servername: false,

    constructor: function(config) {
        qtunes = this;
        prefix = document.location.toString();
//        prefix = "http://localhost:3699/webplayer/static/index.html";        // DEBUG
        prefix = prefix.substring(0, prefix.length - "static/index.html".length);

        this.addEvents({"updated": true});

        playlistPanel = new QTunes.PlaylistPanel({
            qtunes: this
        });

        playPanel = new QTunes.PlayPanel({
            qtunes: this
        });

        trackPanel = new QTunes.TrackPanel({
            qtunes: this,
            gridurl: prefix + "grid"
        });

        rootPlaylist = new QTunes.Playlist({
            qtunes: this,
            id: -1,
            name: "*",
            display: "Music",
            smart: true,
            root: true,
        });

        nowPlaylist = new QTunes.Playlist({
            qtunes: this,
            id: -2,
            name: "nowplaying",
            display: "Now Playing",
            smart: true,
            nowplaying: true,
        });

        QTunes.QTunes.superclass.constructor.call(this, config);
    },

    login: function(username, password) {
        var that = this;
        if (!username) {
            username = "";
        }
        if (!password) {
            password = "";
        }
        qcall("login", { "user":username, "password":password }, function(res) {
//            qtunes.debug("qtunes.login: "+Ext.util.JSON.encode(res));
            if (res.id === 0) {
                var window = new Ext.Window({
                    title: "qTunes Login",
                    layout: 'fit',                 
                    height: 140,
                    width: 260,                            
                    closable: false,
                    resizable: false,                              
                    draggable: false,
                    items: [{
                        xtype: "form",
                        frame: true,
                        ref: ".",
                        width: 260,     
                        labelWidth: 60,         
                        defaults: { width: 165 },
                        items: [{
                            xtype: "textfield",
                            id: "user",
                            fieldLabel: "Username",
                            allowBlank: false,
                            blankText: "User"
                        }, {
                            xtype: "textfield",
                            id: "password",
                            fieldLabel: "Password",
                            inputType: "password",
                            allowBlank: false,
                            blankText: "Enter your password"
                        }],
                        buttons: [{
                            text: 'Login',                         
                            id: "button",
                            ref: "../..",
                            handler: function(b, e) {
                                var form = b.refOwner;
                                if (form.getForm().isValid()) {
                                    that.login(form.findById("user").getValue(), form.findById("password").getValue());
                                    form.refOwner.close();
                                }
                            }
                        }]
                    }]
                });
                window.show();
            } else {
                var ix = 0;
                var oldsession = session;
                if (that.isAllowed("admin", res) && !that.isAllowed("admin")) {
                    // TODO add admin
                } else if (that.isAllowed("admin") && !that.isAllowed("admin", res)) {
                    // TODO remove admin
                }

                session = res;
                that.update(0);
            }
        }, true);
    },

    update: function(lastupdate) {
        var that = this;
        Ext.Ajax.request({
            url: prefix + "status?update="+lastupdate+"&session="+session.id,
            success: function(response) {
                var res = Ext.util.JSON.decode(response.responseText);
                qtunes.debug("qtunes.update "+Ext.util.JSON.encode(res));
                var name, id;
                lastupdate = res.update;

                that.servername = res.name;
                document.title = that.servername;
                if (playlists == null) {
                    playlists = new Object();
                    playlists[rootPlaylist.getId()] = rootPlaylist;
                    playlists[nowPlaylist.getId()] = nowPlaylist;
                    addPlaylist(that, rootPlaylist);
                    addPlaylist(that, nowPlaylist);
                }

                var playlist;
                for (id in res.db.playlists) {
                    if (!playlists[id]) {
                        playlist = new QTunes.Playlist({
                            qtunes: that,
                            id: id,
                            name: res.db.playlists[id].name,
                            display: res.db.playlists[id].name,
                            smart: res.db.playlists[id].smart,
                        });
                        playlists[id] = playlist;
                        addPlaylist(that, playlist);
                    }
                    playlists[id].update(res.db.playlists[id]);
                }
                for (id in playlists) {
                    playlist = playlists[id];
                    if (playlist != rootPlaylist && playlist!= nowPlaylist && !res.db.playlists[id]) {
                        delete playlists[id];
                        removePlaylist(that, playlist);
                    }
                }

                var speaker;
                for (var speakername in res.speakers) {
                    if (!speakers[speakername]) {
                        speaker = new QTunes.Speaker({
                            qtunes: that,
                            name: speakername,
                            display: res.speakers[speakername].name,
                        });
                        speakers[speakername] = speaker;
                        addSpeaker(that, speaker);
                    }
                    speakers[speakername].update(res.speakers[speakername]);
                }
                for (var name in speakers) {
                    speaker = speakers[name];
                    if (!res.speakers[name]) {
                        delete speakers[name];
                        removeSpeaker(that, speaker);
                    }
                }

                var player;
                for (var playername in res.players) {
                    if (!players[playername]) {
                        player = new QTunes.Player({
                            qtunes: that,
                            name: playername,
                            display: res.players[playername].name,
                            playlistname: "#"+playername,
                        });
                        players[playername] = player;
                        addPlayer(that, player);
                    }
                    players[playername].update(res.players[playername]);
                }
                for (var name in players) {
                    player = players[name];
                    if (!res.players[name]) {
                        delete players[name];
                        removePlayer(that, player);
                    }
                }

                if (res.daap) {
                    for (var i=0;i<res.daap.pendingremotes.length;i++) {
                        var remotename = res.daap.pendingremotes[i];
                        this.notifySpecialEvent("remote", "", new function() {
                            var store = that.getAllPlayers()
                            for (var j=0;j<store.length;j++) {
                                store[j] = [store[j].getName(), store[j].getDisplay()];
                            }
                            var popup = new Ext.Window({
                                modal:true,
                                layout:'fit',
                                height: 140,
                                width: 340,                            
                                closable: false,
                                resizable: false,                              
                                draggable: false,
                                items:[{
                                    title: "Pair Remote \""+remotename+"\"",
                                    xtype: "form",
                                    ref: ".",
                                    frame: true,
                                    width: 340,     
                                    labelWidth: 100,         
                                    defaults: { width: 165, allowBlank: false },
                                    items:[{
                                        id: "remotePlayerField",
                                        xtype: 'combo',
                                        fieldLabel: 'Player',
                                        editable: false,
                                        triggerAction: 'all',
                                        typeAhead: false,
                                        mode: 'local',
                                        store: store,
                                    },{
                                        id: "remotePinField",
                                        xtype: 'textfield',
                                        fieldLabel:"PIN",
                                    }],
                                    buttons:[{
                                        text:"Cancel",
                                        ref: "../..",
                                        handler: function() {
                                            popup.close();
                                        }
                                    }, {
                                        text: "Pair",
                                        ref: "../..",
                                        handler: function(b, e) {
                                            var form = b.refOwner;
                                            if (form.getForm().isValid()) {
                                                that.actionAuthorizeRemote(form.findById("remotePlayerField").getValue(), remotename, form.findById("remotePinField").getValue());
                                                form.refOwner.close();
                                            }
                                        }
                                    }]
                                }],
                            });
                            popup.show();
                        });
                    }
                }


                rootPlaylist.update({revision: res.db.revision, size:res.db.size});
                that.fireEvent("updated");
                if (res.error) {
                    that.displayThrowable(res.error);
                }

                that.update(res.update);
            },
            failure: function(response) {
                Ext.Msg.show({
                    title: 'Invalid Session',
                    msg: 'The session has been invalidated. Click OK to reload',
                    buttons: Ext.Msg.OKCANCEL,
                    fn: function(id, text, opt) {
                        if (id == "ok") {
                            window.location.reload();
                        }
                    },
                });
            },

        });
    },

    isAllowed: function(status, funcsession) {
        if (!funcsession) {
            funcsession = this.session;
            if (!funcsession) {
                return false;
            }
        }
        var z = funcsession.id != 0 && (funcsession.allowances.indexOf('*')>=0 || funcsession.allowances[status]) ? true : false;
        return z;
    },

    setPlayer: function(player) {
        playPanel.setPlayer(player);
        trackPanel.setPlaylist(trackPanel.getPlaylist());
    },

    setPlaylist: function(playlist) {
        playlistPanel.setPlaylist(playlist);
        trackPanel.setPlaylist(playlist);
    },

    setFilter: function(filter) {
        trackPanel.setFilter(filter);
    },

    getPlayer: function() {
        return playPanel.getPlayer();
    },

    getNamedPlayer: function(name) {
        return players[name];
    },

    getSpeaker: function(name) {
        return speakers[name];
    },

    getPlaylist: function() {
        return trackPanel.getPlaylist();
    },

    getRootPlaylist: function() {
        return rootPlaylist;
    },

    getNowPlaylist: function() {
        return nowPlaylist;
    },

    getNamedPlaylist: function(name) {
        for (var id in playlists) {
            if (playlists[id].getName() == name) {
                return playlists[name];
            }
        }
        return null;
    },

    getAllPlaylists: function() {
        var n = new Array();
        for (var id in playlists) {
            n[n.length] = playlists[id];
        }
        return n;
    },

    getAllPlayers: function() {
        var n = new Array();
        for (var x in players) {
            n[n.length] = players[x];
        }
        return n;
    },

    getSession: function() {
        return sess
    },

    getPlayPanel: function() {
        return playPanel;
    },

    getPlaylistPanel: function() {
        return playlistPanel;
    },

    getTrackPanel: function() {
        return trackPanel;
    },

    notifySpecialEvent: function(type, tip, callback) {
        this.call(callback);
    },

    speakerSetup: function() {
        var window;
        var speakerform = new Ext.form.FormPanel({
            frame: true,
            ref: ".",
            defaults: { },
            buttons: [{
                text: "Cancel",
                ref: "../..",
                handler: function() {
                    window.close();
                }
            }, {
                text: 'OK',                         
                id: "button",
                ref: "../..",
                handler: function(b, e) {
                    var form = b.refOwner;
                    if (form.getForm().isValid()) {
                        var props = form.getForm().getValues();
                        for (var speakername in speakers) {
                            if (!props[speakername]) {
                                props[speakername] = "";
                            }
                        }
                        qtunes.actionSetSpeakers(props);
                        form.refOwner.close();
                    }
                }
            }]
        });

        var playerarray = [];
        for (var playername in players) {
            var player = players[playername];
            playerarray[playerarray.length] = {
                xtype: 'button',
                name: playername,
                text: player.getDisplay(),
                enableToggle: true,
                pressed: qtunes.getPlayer()==player,
                toggleGroup: 'speakerPlayer',
                allowDepress: false,
                handler: function(b, e) {
                    playername = b.initialConfig.name;
                    player = players[playername];
                    qtunes.debug("setPlayer("+playername+")");
                    qtunes.setPlayer(player);
                }
            };
        }
        playerarray[playerarray.length] = { xtype: 'label', text: '' };

        speakerform.add(new Ext.form.RadioGroup({
            columns: playerarray.length,
            items: playerarray,
        }));

        var workspeakermap = {};
        for (var speakername in speakers) {
            playerarray = [];
            var speaker = speakers[speakername];
            for (var playername in players) {
                var player = players[playername];
                if (player.getSpeakers().indexOf(speakername) >= 0) {
                    workspeakermap[speakername] = playername;
                }
                playerarray[playerarray.length] = {
                    name: speakername,
                    playername: playername,
                    inputValue: player.getName(),
                    xtype: 'checkbox',
                    checked: workspeakermap[speakername] == playername,
                    anchor: 'center',
                    handler: function(b, checked) {
                        speakername = b.initialConfig.name;
                        playername = b.initialConfig.playername;
                        if (checked && workspeakermap[speakername]) {
                            var speakerdisplay = speakers[speakername].getDisplay();
                            if (!speakerdisplay) {
                                speakerdisplay = speakername;
                            }
                            var playerdisplay = players[workspeakermap[speakername]].getDisplay();
                            if (!playerdisplay) {
                                playerdisplay = workspeakermap[speakername];
                            }
                            Ext.Msg.alert("Error", "Speaker \""+speakerdisplay+"\" is already assigned to \""+playerdisplay+"\"");
                            b.setValue(false);
                        } else if (checked) {
                            workspeakermap[speakername] = playername;
                        } else if (workspeakermap[speakername] == playername) {
                            workspeakermap[speakername] = null;
                        }
                    }
                };
            }
            playerarray[playerarray.length] = {
                name: 'volume',
                id: 'vol_'+speakername,
                xtype: 'slider',
                minValue: -20,
                maxValue: 20,
                value: speaker.getVolumeAdjustment(),
                listeners: {
                    changecomplete: function(item, value, thumb) {
                        var speaker = speakers[item.id.substring(4)];
                        qtunes.actionSetSpeakerVolume(speaker, value);
                    }
                },
            };
            speakerform.add(new Ext.form.RadioGroup({
                id: speakername,
                fieldLabel: speaker.getDisplay(),
                columns: playerarray.length,
                items: playerarray,
            }));
        }

        window = new Ext.Window({
            title: "Speaker Setup",
            modal: true,
            layout: 'fit',                 
            height: 200,
            width: 400,                            
            closable: false,
            resizable: true,                              
            draggable: true,
            items: [speakerform]
        });
        window.show();
    },

    //-------------------------------------------------------------------------------

    actionPlayTracks: function(player, tracks) {
        if (player.validatePlay()) {
            qtunes.debug("ACTION: playTracks("+player.getName()+", "+tracks+")");
            qcall("play", {player: player.getName(), tracks: tracks.join(",") });
        }
    },

    actionEnqueueTracks: function(player, tracks) {
        if (player.validatePlay()) {
            qtunes.debug("ACTION: enqueueTracks("+player.getName()+", "+tracks+")");
            qcall("enqueue", {player: player.getName(), tracks: tracks.join(",") });
        }
    },

    actionDequeueTracks: function(player, indices) {
        qtunes.debug("ACTION: dequeueTracks("+player.getName()+", "+indices+")");
        qcall("dequeue", {player: player.getName(), indices: indices.join(",") });
    },

    actionPlayPlaylist: function(player, playlist, sortfield, sortdir, filter) {
        if (player.validatePlay()) {
            if (!sortfield) sortfield = "";
            if (!sortdir) sortdir = "";
            if (!filter) filter = "";
            qtunes.debug("ACTION: playPlaylist("+player.getName()+", "+playlist.getName()+", "+sortfield+", "+sortdir+", "+filter+")");
            qcall("play", {player: player.getName(), playlist: playlist.getName(), sortfield: sortfield, sortdir: sortdir, filter: filter });
        }
    },

    actionEnqueuePlaylist: function(player, playlist, sortfield, sortdir, filter) {
        if (player.validatePlay()) {
            if (!sortfield) sortfield = "";
            if (!sortdir) sortdir = "";
            if (!filter) filter = "";
            qtunes.debug("ACTION: enqueuePlaylist("+player.getName()+", "+playlist.getName()+", "+sortfield+", "+sortdir+", "+filter+")");
            qcall("enqueue", {player: player.getName(), playlist: playlist.getName(), sortfield: sortfield, sirtdir: sortdir, filter: filter });
        }
    },

    actionDeletePlaylist: function(playlist) {
        qtunes.debug("ACTION: deletePlaylist("+playlist.getName()+")");
        qcall("deleteplaylist", {playlist: playlist.getName() });
    },

    actionRenamePlaylist: function(playlist, newname) {
        qtunes.debug("ACTION: renamePlaylist("+playlist.getName()+", "+newname+")");
        qcall("renameplaylist", {from: playlist.getName(), to: newname });
    },

    actionDeleteTracks: function(playlist, indices) {
        qtunes.debug("ACTION: deleteTracks("+playlist.getName()+", "+indices+")");
        qcall("deletetracks", {playlist: playlist.getName(), indices: indices.join(",") });
    },

    actionCreatePlaylist: function(name, tracks, replace) {
        qtunes.debug("ACTION: createPlaylist("+name+", "+tracks+", "+replace+")");
        qcall("createplaylist", {playlist: name, tracks: tracks.join(","), replace: replace });
    },

    actionCreateSmartPlaylist: function(name, query) {
        qtunes.debug("ACTION: createSmartPlaylist("+name+", "+query+")");
        qcall("createplaylist", {playlist: name, query: query });
    },

    actionEditSmartPlaylist: function(name, query) {
        qtunes.debug("ACTION: editSmartPlaylist("+name+", "+query+")");
        qcall("editplaylist", {playlist: name, query: query });
    },

    actionAddTracks: function(playlist, tracks) {
        qtunes.debug("ACTION: addTracks("+playlist.getName()+", "+tracks+")");
        qcall("addtracks", {playlist: playlist.getName(), tracks: tracks.join(",") });
    },

    actionSetPaused: function(player, paused) {
        if (player.validatePlay()) {
            qtunes.debug("ACTION: pause("+player.getName()+","+paused+")");
            qcall("pause", {player: player.getName(), paused: paused ? "1" : "0" });
        }
    },

    actionSeek: function(player, ms) {
        qtunes.debug("ACTION: seek("+player.getName()+", "+ms+")");
        qcall("seek", {player: player.getName(), ms: ms });
    },

    actionSetVolume: function(player, volume) {
        qtunes.debug("ACTION: setVolume("+player.getName()+", "+volume+")");
        qcall("setvolume", {player: player.getName(), volume: volume });
    },

    actionSetSpeakerVolume: function(speaker, volume) {
        qtunes.debug("ACTION: setSpeakerVolume("+speaker.getName()+", "+volume+")");
        qcall("setvolume", {speaker: speaker.getName(), volume: volume });
    },

    actionPlayNext: function(player) {
        if (player.validatePlay()) {
            qtunes.debug("ACTION: playNext("+player.getName()+")");
            qcall("playnext", {player: player.getName() });
        }
    },

    actionPlayPrevious: function(player) {
        if (player.validatePlay()) {
            qtunes.debug("ACTION: playPrevious("+player.getName()+")");
            qcall("playprevious", {player: player.getName() });
        }
    },

    actionSkipToTrack: function(player, index) {
        qtunes.debug("ACTION: skipToTrack("+player.getName()+", "+index+")");
        qcall("setplayindex", {player: player.getName(), index: index });
    },

    actionSetShuffle: function(player, shuffle) {
        qtunes.debug("ACTION: setShuffle("+player.getName()+", "+shuffle+")");
        qcall("setshuffle", {player: player.getName(), shuffle: shuffle ? true : false });
    },

    actionSetRepeat: function(player, repeat) {
        qtunes.debug("ACTION: setRepeat("+player.getName()+", "+repeat+")");
        qcall("setrepeat", {player: player.getName(), repeat: repeat });
    },

    actionAuthorizeRemote: function(player, remote, pin) {
        qtunes.debug("ACTION: authorizeRemote("+player+", "+remote+", "+pin+")");
        qcall("authorizeremote", {player: player, remote: remote, pin: pin });
    },

    actionSetSpeakers: function(props) {
        qtunes.debug("ACTION: setSpeakers("+Ext.util.JSON.encode(props)+")");
        qcall("setspeakers", props);
    },

    actionClear: function(player) {
        qtunes.debug("ACTION: clear("+player.getName()+")");
        qcall("clear", {player: player.getName()});
    },

    debug: function(text) {
        if (console) {
            console.log(text);
        }
    },

    displayThrowable: function(e) {
        var msg = e.message + "\n\n";
        for (i=0;i<e.trace.length;i++) {
            msg += "  "+e.trace[i]+"\n";
        }
        Ext.Msg.alert("Server Error ("+e.type+")", msg);
    },

};

}()));
