Ext.ns("QTunes");
QTunes.TrackPanel = Ext.extend(Ext.ux.grid.livegrid.GridPanel, (function() {

var qtunes;
var playlist;
var sortinfo;
var lastplayingindex = -1;
var playlistUpdateListener;

var makePopup = function(me, qtunes, playlist) {
    var popup = new Ext.menu.Menu();

    if (!playlist.isNowPlaying()) {
        popup.addItem(new Ext.menu.Item({
            text: "Play",
            handler: function(n, e) {
                var tracks = me.getSelectedTracks();
                qtunes.actionPlayTracks(qtunes.getPlayer(), tracks);
            }
        }));
        popup.addItem(new Ext.menu.Item({
            text: "Enqueue",
            handler: function(n, e) {
                var tracks = me.getSelectedTracks();
                qtunes.actionEnqueueTracks(qtunes.getPlayer(), tracks);
            }
        }));
    }
    if (playlist.isNowPlaying() || !playlist.isSmart()) {
        popup.addItem(new Ext.menu.Item({
            text: "Delete",
            handler: function(n, e) {
                me.deleteSelectedTracks(false);
            }
        }));
    }

    var submenu = new Ext.menu.Menu({});
    popup.addItem(new Ext.menu.Item({
        text: "Add to Playlist",
        menu: submenu,
    }));
    submenu.addItem(new Ext.menu.Item({
        text: "-- New --",
        handler: function(n, e) {
            Ext.Msg.prompt("Name", "Playlist Name:", function(btn, text) {
                if (btn == 'ok' && text.length > 0) {
                    if (qtunes.getNamedPlaylist(text) != null) {
                        Ext.Msg.alert("Error", "Playlist \""+text+"\" already exists");
                    } else {
                        var tracks = me.getSelectedTracks();
                        qtunes.actionCreatePlaylist(text, tracks);
                    }
                }
            });
        }
    }));
    var allplaylists = qtunes.getAllPlaylists();
    for (var i=0;i<allplaylists.length;i++) {
        var tempplaylist = allplaylists[i];
        if (!tempplaylist.isRoot() && !tempplaylist.isNowPlaying() && tempplaylist != playlist) {
            var item = new Ext.menu.Item({
                text: tempplaylist.getDisplay(),
                handler: function(n, e) {
                    var tracks = me.getSelectedTracks();
                    qtunes.actionAddTracks(this.playlist, tracks);
                }
            });
            item.playlist = tempplaylist;
            submenu.addItem(item);
        }
    }
    return popup;
};

return {
    /**
    * Required:
    *   gridurl: 'http://localhost/webplayer/grid'
    */
    constructor: function(config) {
        qtunes = config.qtunes;
        var that = this;

        config = Ext.apply({
            enableDragDrop: false,
            enableColumnMove: true,
            cm: new Ext.grid.ColumnModel({}),
            loadMask: {
                msg: 'Loading...'
            },
            stripeRows: true,
            store: new Ext.ux.grid.livegrid.Store({
                url: config.gridurl,
                bufferSize: 300,
                reader: new Ext.ux.grid.livegrid.JsonReader({
                    root: 'data',
                    versionProperty: 'version',
                    totalProperty: 'count',
                    id: 'n',
                  },[
                    { name: 'id' },
                    { name: 'n', sortType: Ext.data.SortTypes.asInt },
                    { name: 'Title', sortType: Ext.data.SortTypes.asUCString },
                    { name: 'Artist', sortType: Ext.data.SortTypes.asUCString },
                    { name: 'AlbumArtist', sortType: Ext.data.SortTypes.asUCString },
                    { name: 'Album',   sortType: Ext.data.SortTypes.asUCString },
                    { name: 'Track' },
                    { name: 'Disc' },
                    { name: 'Year' },
                    { name: 'LastModified' },
                    { name: 'Group' },
                ]),
                sortInfo: {field: 'Album', direction: 'ASC'}
            }),
            selModel : new Ext.ux.grid.livegrid.RowSelectionModel(),
            view: new Ext.ux.grid.livegrid.GridView({
                nearLimit: 100,
                loadMask: { msg: 'Buffering. Please wait...' },
                getRowClass: function(record, rowIndex, rp, ds) {
                    if (playlist && playlist.isNowPlaying()) {
                        if (rowIndex==qtunes.getPlayer().getPlaylistIndex()) {
                            return "nowplaying-now";
                        } else {
                            return rowIndex%2==0 ? "nowplaying-even" : "nowplaying-odd";
                        }
                    }
                    return "";
                }
            }),
            listeners: {
                rowcontextmenu: function(grid, rowIndex, e) {
                    e.stopEvent();
                    this.getSelectionModel().selectRow(rowIndex, true);
                    var popup = makePopup(that, qtunes, playlist);
                    popup.showAt(e.getXY());
                },
                rowdblclick: function(grid, rowIndex, e) {
                    if (playlist.isNowPlaying()) {
                        qtunes.actionSkipToTrack(qtunes.getPlayer(), rowIndex);
                    } else {
                        var track = this.getStore().getAt(rowIndex).get("id");
                        qtunes.actionPlayTracks(qtunes.getPlayer(), [track]);
                    }
                },
                render: function(c) {
                    c.getEl().on('keypress', function(e) {
                        if (e.getKey() == Ext.EventObject.BACKSPACE) {
                            this.deleteSelectedTracks(true);
                        } else if (e.getKey() == Ext.EventObject.CTRL | Ext.EventObject.A) {
                            this.getSelectionModel().selectAll();
                        } else {
                            return;
                        }
                        e.preventDefault();
                        e.stopPropagation();
                    }, c);
                },
                sortchange: function(grid, newsortinfo) {
                    sortinfo = newsortinfo;
                },
            }
        }, config);
        QTunes.TrackPanel.superclass.constructor.call(this, config);

        playlistUpdateListener = function() {
            that.getView().updateLiveRows(that.getView().rowIndex, true, true);
        };

    },

    setPlaylist: function(newplaylist) {
        if (newplaylist != playlist || (newplaylist.isNowPlaying() && this.nowplayingplayer!=qtunes.getPlayer())) {
            if (playlist) {
                playlist.un("update", playlistUpdateListener);
            }
            qtunes.debug("TrackPanel.setPlaylist("+newplaylist+")");
            playlist = newplaylist;
            playlist.on("update", playlistUpdateListener);
            if (playlist.isNowPlaying()) {
                this.getStore().setBaseParam("playlist", "#"+qtunes.getPlayer().getName());
                this.nowplayingplayer = qtunes.getPlayer();
            } else {
                this.getStore().setBaseParam("playlist", playlist.getName());
            }
            this.getSelectionModel().clearSelections(true);
            var columns = [
                {header: "#", align: 'left', width: 40, sortable: true, dataIndex: 'n'},
                {header: "Title", align: 'left', width: 160, sortable: true, dataIndex: 'Title'},
                {header: "Artist", align: 'left', width: 160, sortable: true, dataIndex: 'Artist'},
                {header: "Album Artist", align: 'left', width: 160, sortable: true, dataIndex: 'AlbumArtist'},
                {header: "Album", align: 'left',  width: 160, sortable: true, dataIndex: 'Album'},
                {header: "Track", align: 'left',  width: 60, sortable: false, dataIndex: 'Track'},
                {header: "Disc", align: 'left',  width: 60, sortable: false, dataIndex: 'Disc'},
                {header: "Updated", align: 'left',  width: 60, sortable: false, dataIndex: 'LastModified'},
                {header: "Year", align: 'left',  width: 60, sortable: false, dataIndex: 'Year'},
                {header: "Group", align: 'left',  width: 60, sortable: false, dataIndex: 'Group'},
            ];
            if (playlist.isNowPlaying()) {
                for (var i=0;i<columns.length;i++) {
                    columns[i].sortable = false;
                }
                this.getStore().sort([{field: 'n', direction: 'ASC'}]);
            }
            this.getColumnModel().setConfig(columns);
            this.getView().reset(true);
        }
    },

    getSelectedTracks: function() {
        var records = this.getSelectionModel().getSelections();
        for (var i=0;i<records.length;i++) {
            records[i] = records[i].get("id");
        }
        return records;
    },

    getSelectedIndices: function() {
        var records = this.getSelectionModel().getSelections();
        for (var i=0;i<records.length;i++) {
            records[i] = records[i].get("n") - 1;
        }
        return records;
    },

    getSort: function() {
        return sortinfo;
    },

    getPlaylist: function() {
        return playlist;
    },

    getFilter: function() {
        return this.getStore().baseParams.filter;
    },

    setFilter: function(newfilter) {
        if (newfilter != this.getStore().baseParams.filter) {
            this.getStore().setBaseParam("filter", newfilter);
            this.getView().reset(true);
        }
    },

    deleteSelectedTracks: function(prompt) {
        var indices = this.getSelectedIndices();
        var player = qtunes.getPlayer();
        if (indices.length > 0 && (playlist.isNowPlaying() || !playlist.isSmart())) {
            this.getSelectionModel().clearSelections();
            if (prompt) {
                Ext.Msg.show({
                    title: 'Delete tracks',
                    msg: 'Delete selected tracks from playlist?',
                    buttons: Ext.Msg.YESNO,
                    fn: function(id, text, opt) {
                        if (id == Ext.Msg.YES) {
                            if (playlist.isNowPlaying()) {
                                qtunes.actionDequeueTracks(player, indices);
                            } else {
                                qtunes.actionDeleteTracks(playlist, indices);
                            }
                        }
                    },
                    icon: Ext.MessageBox.QUESTION,
                });
            } else {
                if (playlist.isNowPlaying()) {
                    qtunes.actionDequeueTracks(player, indices);
                } else {
                    qtunes.actionDeleteTracks(playlist, indices);
                }
            }
        }
        this.getView().reset(true);
    },

};

}()));
